package com.app.yourrecipesapp.fragments;

import android.app.Activity;
import android.content.Intent;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.StaggeredGridLayoutManager;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;
import androidx.viewpager.widget.ViewPager;

import com.app.yourrecipesapp.BuildConfig;
import com.app.yourrecipesapp.R;
import com.app.yourrecipesapp.activities.ActivityCategoryDetail;
import com.app.yourrecipesapp.activities.ActivityRecipeDetail;
import com.app.yourrecipesapp.activities.ActivityRecipeList;
import com.app.yourrecipesapp.activities.MainActivity;
import com.app.yourrecipesapp.adapters.AdapterFeatured;
import com.app.yourrecipesapp.adapters.AdapterHomeCategory;
import com.app.yourrecipesapp.adapters.AdapterHomeRecipes;
import com.app.yourrecipesapp.callbacks.CallbackHome;
import com.app.yourrecipesapp.config.AppConfig;
import com.app.yourrecipesapp.models.Category;
import com.app.yourrecipesapp.models.Recipe;
import com.app.yourrecipesapp.rests.RestAdapter;
import com.app.yourrecipesapp.utils.AdsPref;
import com.app.yourrecipesapp.utils.Constant;
import com.app.yourrecipesapp.utils.NativeTemplateStyle;
import com.app.yourrecipesapp.utils.SharedPref;
import com.app.yourrecipesapp.utils.TemplateView;
import com.app.yourrecipesapp.utils.Tools;
import com.duolingo.open.rtlviewpager.RtlViewPager;
import com.facebook.ads.AdError;
import com.facebook.ads.AdOptionsView;
import com.facebook.ads.NativeAd;
import com.facebook.ads.NativeAdLayout;
import com.facebook.ads.NativeAdListener;
import com.facebook.shimmer.ShimmerFrameLayout;
import com.google.android.gms.ads.AdListener;
import com.google.android.gms.ads.AdLoader;
import com.google.android.gms.ads.MobileAds;
import com.google.android.gms.ads.formats.MediaView;
import com.google.android.material.tabs.TabLayout;
import com.startapp.sdk.ads.nativead.NativeAdDetails;
import com.startapp.sdk.ads.nativead.NativeAdPreferences;
import com.startapp.sdk.ads.nativead.StartAppNativeAd;
import com.startapp.sdk.adsbase.Ad;
import com.startapp.sdk.adsbase.adlisteners.AdEventListener;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

import static com.app.yourrecipesapp.utils.Constant.ADMOB;
import static com.app.yourrecipesapp.utils.Constant.AD_STATUS_ON;
import static com.app.yourrecipesapp.utils.Constant.FAN;
import static com.app.yourrecipesapp.utils.Constant.STARTAPP;
import static com.app.yourrecipesapp.utils.Constant.STARTAPP_IMAGE_LARGE;
import static com.app.yourrecipesapp.utils.Constant.STARTAPP_IMAGE_XSMALL;

public class FragmentHome extends Fragment {

    private View root_view, parent_view;;
    private SwipeRefreshLayout swipe_refresh;
    private Call<CallbackHome> callbackCall = null;
    private View lyt_main_content;
    private Runnable runnableCode = null;
    private Handler handler = new Handler();
    private ViewPager view_pager_featured;
    private RtlViewPager view_pager_featured_rtl;
    public static final String EXTRA_OBJC = "key.EXTRA_OBJC";
    private ShimmerFrameLayout lyt_shimmer;
    private TemplateView native_template;
    private MediaView mediaView;
    private StartAppNativeAd startAppNativeAd;
    private NativeAdDetails nativeAd = null;
    SharedPref sharedPref;
    AdsPref adsPref;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        if (AppConfig.ENABLE_RTL_MODE) {
            root_view = inflater.inflate(R.layout.fragment_home_rtl, container, false);
        } else {
            root_view = inflater.inflate(R.layout.fragment_home, container, false);
        }
        parent_view = getActivity().findViewById(R.id.lyt_content);

        sharedPref = new SharedPref(getActivity());
        adsPref = new AdsPref(getActivity());

        swipe_refresh = root_view.findViewById(R.id.swipe_refresh);
        swipe_refresh.setColorSchemeResources(R.color.colorPrimary);
        lyt_main_content = root_view.findViewById(R.id.lyt_home_content);
        lyt_shimmer = root_view.findViewById(R.id.shimmer_view_container);
        native_template = root_view.findViewById(R.id.native_template);
        mediaView = root_view.findViewById(R.id.media_view);

        // on swipe list
        swipe_refresh.setOnRefreshListener(this::requestAction);

        requestAction();

        return root_view;
    }

    private void requestAction() {
        showFailedView(false, "");
        swipeProgress(true);
        new Handler().postDelayed(this::requestHomeData, Constant.DELAY_TIME);
    }

    private void requestHomeData() {
        this.callbackCall = RestAdapter.createAPI().getHome(AppConfig.API_KEY);
        this.callbackCall.enqueue(new Callback<CallbackHome>() {
            public void onResponse(Call<CallbackHome> call, Response<CallbackHome> response) {
                CallbackHome responseHome = response.body();
                if (responseHome == null || !responseHome.status.equals("ok")) {
                    onFailRequest();
                    return;
                }
                displayData(responseHome);
                swipeProgress(false);
                lyt_main_content.setVisibility(View.VISIBLE);
                initAdNetwork();
            }

            public void onFailure(Call<CallbackHome> call, Throwable th) {
                Log.e("onFailure", th.getMessage());
                if (!call.isCanceled()) {
                    onFailRequest();
                }
            }
        });
    }

    private void onFailRequest() {
        swipeProgress(false);
        if (Tools.isConnect(getActivity())) {
            showFailedView(true, getString(R.string.failed_text));
        } else {
            showFailedView(true, getString(R.string.failed_text));
        }
    }

    private void showFailedView(boolean show, String message) {
        View lyt_failed = root_view.findViewById(R.id.lyt_failed_home);
        ((TextView) root_view.findViewById(R.id.failed_message)).setText(message);
        if (show) {
            lyt_failed.setVisibility(View.VISIBLE);
            lyt_main_content.setVisibility(View.GONE);
        } else {
            lyt_failed.setVisibility(View.GONE);
            lyt_main_content.setVisibility(View.VISIBLE);
        }
        root_view.findViewById(R.id.failed_retry).setOnClickListener(view -> requestAction());
    }

    private void swipeProgress(final boolean show) {
        if (!show) {
            swipe_refresh.setRefreshing(show);
            lyt_shimmer.setVisibility(View.GONE);
            lyt_shimmer.stopShimmer();
            lyt_main_content.setVisibility(View.VISIBLE);

            return;
        }
        swipe_refresh.post(() -> {
            swipe_refresh.setRefreshing(show);
            lyt_shimmer.setVisibility(View.VISIBLE);
            lyt_shimmer.startShimmer();
            lyt_main_content.setVisibility(View.GONE);
        });
    }

    private void startAutoSlider(final int position) {

        if (AppConfig.ENABLE_RTL_MODE) {
            if (this.runnableCode != null) {
                this.handler.removeCallbacks(this.runnableCode);
            }
            this.runnableCode = () -> {
                int currentItem = view_pager_featured_rtl.getCurrentItem() + 1;
                if (currentItem >= position) {
                    currentItem = 0;
                }
                view_pager_featured_rtl.setCurrentItem(currentItem);
                handler.postDelayed(FragmentHome.this.runnableCode, AppConfig.AUTO_SLIDER_DURATION);
            };
            handler.postDelayed(this.runnableCode, AppConfig.AUTO_SLIDER_DURATION);
        } else {
            if (this.runnableCode != null) {
                this.handler.removeCallbacks(this.runnableCode);
            }
            this.runnableCode = () -> {
                int currentItem = view_pager_featured.getCurrentItem() + 1;
                if (currentItem >= position) {
                    currentItem = 0;
                }
                view_pager_featured.setCurrentItem(currentItem);
                handler.postDelayed(FragmentHome.this.runnableCode, AppConfig.AUTO_SLIDER_DURATION);
            };
            handler.postDelayed(this.runnableCode, AppConfig.AUTO_SLIDER_DURATION);
        }

    }

    private void displayData(CallbackHome responseHome) {
        displayFeatured(responseHome.featured);
        displayCategory(responseHome.category);
        displayRecent(responseHome.recent);
        displayVideos(responseHome.videos);
        displayData();
    }

    public void initAdNetwork() {
        if (adsPref.getAdStatus().equals(AD_STATUS_ON) && adsPref.getAdType().equals(ADMOB)) {
            MobileAds.initialize(getActivity(), adsPref.getAdMobAppId());
            if (!adsPref.getAdMobNativeId().equals("0")) {
                AdLoader adLoader = new AdLoader.Builder(getActivity(), adsPref.getAdMobNativeId())
                        .forUnifiedNativeAd(unifiedNativeAd -> {
                            if (sharedPref.getIsDarkTheme()) {
                                ColorDrawable colorDrawable = new ColorDrawable(ContextCompat.getColor(getActivity(), R.color.colorBackgroundDark));
                                NativeTemplateStyle styles = new NativeTemplateStyle.Builder().withMainBackgroundColor(colorDrawable).build();
                                native_template.setStyles(styles);
                            } else {
                                ColorDrawable colorDrawable = new ColorDrawable(ContextCompat.getColor(getActivity(), R.color.colorBackgroundLight));
                                NativeTemplateStyle styles = new NativeTemplateStyle.Builder().withMainBackgroundColor(colorDrawable).build();
                                native_template.setStyles(styles);
                            }
                            mediaView.setImageScaleType(ImageView.ScaleType.CENTER_CROP);
                            native_template.setNativeAd(unifiedNativeAd);
                            native_template.setVisibility(View.VISIBLE);
                        })
                        .withAdListener(new AdListener() {
                            @Override
                            public void onAdFailedToLoad(int errorCode) {
                                native_template.setVisibility(View.GONE);
                            }
                        })
                        .build();
                adLoader.loadAd(Tools.getAdRequest(getActivity()));
            }
        } else if (adsPref.getAdStatus().equals(AD_STATUS_ON) && adsPref.getAdType().equals(FAN)) {
            if (!adsPref.getFanNativeUnitId().equals("0")) {
                final NativeAd nativeAd;
                final RelativeLayout lyt_fan_native = root_view.findViewById(R.id.lyt_fan_native);
                final NativeAdLayout nativeAdLayout = root_view.findViewById(R.id.native_ad_container);
                if (BuildConfig.DEBUG) {
                    nativeAd = new NativeAd(getActivity(), "IMG_16_9_APP_INSTALL#" + adsPref.getFanNativeUnitId());
                } else {
                    nativeAd = new NativeAd(getActivity(), adsPref.getFanNativeUnitId());
                }
                NativeAdListener nativeAdListener = new NativeAdListener() {
                    @Override
                    public void onMediaDownloaded(com.facebook.ads.Ad ad) {

                    }

                    @Override
                    public void onError(com.facebook.ads.Ad ad, AdError adError) {

                    }

                    @Override
                    public void onAdLoaded(com.facebook.ads.Ad ad) {
                        // Race condition, load() called again before last ad was displayed
                        lyt_fan_native.setVisibility(View.VISIBLE);
                        if (nativeAd != ad) {
                            return;
                        }
                        // Inflate Native Ad into Container
                        //inflateAd(nativeAd);
                        nativeAd.unregisterView();
                        // Add the Ad view into the ad container.
                        LayoutInflater inflater = LayoutInflater.from(getActivity());
                        // Inflate the Ad view.  The layout referenced should be the one you created in the last step.
                        LinearLayout nativeAdView = (LinearLayout) inflater.inflate(R.layout.gnt_fan_medium_template, nativeAdLayout, false);
                        nativeAdLayout.addView(nativeAdView);

                        // Add the AdOptionsView
                        LinearLayout adChoicesContainer = nativeAdView.findViewById(R.id.ad_choices_container);
                        AdOptionsView adOptionsView = new AdOptionsView(getActivity(), nativeAd, nativeAdLayout);
                        adChoicesContainer.removeAllViews();
                        adChoicesContainer.addView(adOptionsView, 0);

                        // Create native UI using the ad metadata.
                        TextView nativeAdTitle = nativeAdView.findViewById(R.id.native_ad_title);
                        com.facebook.ads.MediaView nativeAdMedia = nativeAdView.findViewById(R.id.native_ad_media);
                        TextView nativeAdSocialContext = nativeAdView.findViewById(R.id.native_ad_social_context);
                        TextView nativeAdBody = nativeAdView.findViewById(R.id.native_ad_body);
                        TextView sponsoredLabel = nativeAdView.findViewById(R.id.native_ad_sponsored_label);
                        Button nativeAdCallToAction = nativeAdView.findViewById(R.id.native_ad_call_to_action);
                        LinearLayout ad_unit = nativeAdView.findViewById(R.id.ad_unit);

                        if (sharedPref.getIsDarkTheme()) {
                            nativeAdTitle.setTextColor(ContextCompat.getColor(getActivity(), R.color.colorWhite));
                            nativeAdSocialContext.setTextColor(ContextCompat.getColor(getActivity(), R.color.colorWhite));
                            sponsoredLabel.setTextColor(ContextCompat.getColor(getActivity(), R.color.sub_text_color));
                            nativeAdBody.setTextColor(ContextCompat.getColor(getActivity(), R.color.sub_text_color));
                        }

                        // Set the Text.
                        nativeAdTitle.setText(nativeAd.getAdvertiserName());
                        nativeAdBody.setText(nativeAd.getAdBodyText());
                        nativeAdSocialContext.setText(nativeAd.getAdSocialContext());
                        nativeAdCallToAction.setVisibility(nativeAd.hasCallToAction() ? View.VISIBLE : View.INVISIBLE);
                        nativeAdCallToAction.setText(nativeAd.getAdCallToAction());
                        sponsoredLabel.setText(nativeAd.getSponsoredTranslation());

                        // Create a list of clickable views
                        List<View> clickableViews = new ArrayList<>();
                        clickableViews.add(nativeAdTitle);
                        clickableViews.add(ad_unit);
                        clickableViews.add(nativeAdCallToAction);

                        // Register the Title and CTA button to listen for clicks.
                        nativeAd.registerViewForInteraction(nativeAdView, nativeAdMedia, clickableViews);

                    }

                    @Override
                    public void onAdClicked(com.facebook.ads.Ad ad) {

                    }

                    @Override
                    public void onLoggingImpression(com.facebook.ads.Ad ad) {

                    }
                };

                NativeAd.NativeLoadAdConfig loadAdConfig = nativeAd.buildLoadAdConfig().withAdListener(nativeAdListener).build();
                nativeAd.loadAd(loadAdConfig);
            }
        } else if (adsPref.getAdStatus().equals(AD_STATUS_ON) && adsPref.getAdType().equals(STARTAPP)) {
            if (!adsPref.getStartappAppID().equals("0")) {
                View lyt_startapp_native = root_view.findViewById(R.id.startapp_native_template);
                ImageView startapp_native_image = root_view.findViewById(R.id.startapp_native_image);
                ImageView startapp_native_secondary_image = root_view.findViewById(R.id.startapp_native_secondary_image);
                TextView startapp_native_title = root_view.findViewById(R.id.startapp_native_title);
                TextView startapp_native_description = root_view.findViewById(R.id.startapp_native_description);
                Button startapp_native_button = root_view.findViewById(R.id.startapp_native_button);
                startapp_native_button.setOnClickListener(v1 -> lyt_startapp_native.performClick());

                startAppNativeAd = new StartAppNativeAd(getActivity());
                NativeAdPreferences nativePrefs = new NativeAdPreferences()
                        .setAdsNumber(1)
                        .setAutoBitmapDownload(true)
                        .setPrimaryImageSize(STARTAPP_IMAGE_LARGE)
                        .setSecondaryImageSize(STARTAPP_IMAGE_XSMALL);
                AdEventListener adListener = new AdEventListener() {
                    @Override
                    public void onReceiveAd(Ad arg0) {
                        ArrayList<NativeAdDetails> nativeAdsList = startAppNativeAd.getNativeAds();
                        if (nativeAdsList.size() > 0) {
                            nativeAd = nativeAdsList.get(0);
                        }
                        if (nativeAd != null) {;
                            startapp_native_image.setImageBitmap(nativeAd.getImageBitmap());
                            startapp_native_secondary_image.setImageBitmap(nativeAd.getSecondaryImageBitmap());
                            startapp_native_title.setText(nativeAd.getTitle());
                            startapp_native_description.setText(nativeAd.getDescription());
                            startapp_native_button.setText(nativeAd.isApp() ? "Install" : "Open");
                            nativeAd.registerViewForInteraction(lyt_startapp_native);
                        }
                        lyt_startapp_native.setVisibility(View.VISIBLE);
                    }

                    @Override
                    public void onFailedToReceiveAd(Ad arg0) {
                        lyt_startapp_native.setVisibility(View.GONE);
                    }
                };
                startAppNativeAd.loadAd(nativePrefs, adListener);
            }
        }
    }

    private void displayData() {

        ((TextView) root_view.findViewById(R.id.txt_title_category)).setText(getResources().getString(R.string.home_title_category));
        ((TextView) root_view.findViewById(R.id.txt_title_recent)).setText(getResources().getString(R.string.home_title_recent));
        ((TextView) root_view.findViewById(R.id.txt_title_videos)).setText(getResources().getString(R.string.home_title_videos));

        ((ImageView) root_view.findViewById(R.id.img_arrow_category)).setImageResource(R.drawable.ic_arrow_next);
        ((ImageView) root_view.findViewById(R.id.img_arrow_recent)).setImageResource(R.drawable.ic_arrow_next);
        ((ImageView) root_view.findViewById(R.id.img_arrow_videos)).setImageResource(R.drawable.ic_arrow_next);

        root_view.findViewById(R.id.ripple_more_category).setOnClickListener(view -> {
            Activity act = getActivity();
            if (act instanceof MainActivity) {
                ((MainActivity) act).selectFragmentCategory();
            }
        });

        root_view.findViewById(R.id.ripple_recent_more).setOnClickListener(view -> {
            Intent intent = new Intent(getActivity(), ActivityRecipeList.class);
            intent.putExtra("title", getResources().getString(R.string.home_title_recent));
            intent.putExtra("filter", 1);
            startActivity(intent);
        });

        root_view.findViewById(R.id.ripple_videos_more).setOnClickListener(view -> {
            Intent intent = new Intent(getActivity(), ActivityRecipeList.class);
            intent.putExtra("title", getResources().getString(R.string.home_title_videos));
            intent.putExtra("filter", 2);
            startActivity(intent);
        });

    }

    private void displayFeatured(final List<Recipe> list) {

        if (AppConfig.ENABLE_RTL_MODE) {
            view_pager_featured_rtl = root_view.findViewById(R.id.view_pager_featured_rtl);
            final AdapterFeatured adapter = new AdapterFeatured(getActivity(), list);
            final LinearLayout lyt_featured = root_view.findViewById(R.id.lyt_featured);
            view_pager_featured_rtl.setAdapter(adapter);
            view_pager_featured_rtl.setOffscreenPageLimit(4);

            if (list.size() > 0) {
                lyt_featured.setVisibility(View.VISIBLE);
            } else {
                lyt_featured.setVisibility(View.GONE);
            }

            view_pager_featured_rtl.addOnPageChangeListener(new ViewPager.SimpleOnPageChangeListener() {

                public void onPageSelected(int position) {
                    super.onPageSelected(position);
                    if (position < list.size()) {

                    }
                }
            });
            adapter.setOnItemClickListener((view, obj) -> {
                Intent intent = new Intent(getActivity(), ActivityRecipeDetail.class);
                intent.putExtra(Constant.EXTRA_OBJC, obj);
                startActivity(intent);
            });

            TabLayout tabLayout = root_view.findViewById(R.id.tabDots);
            tabLayout.setupWithViewPager(view_pager_featured_rtl, true);

            startAutoSlider(list.size());
        } else {
            view_pager_featured = root_view.findViewById(R.id.view_pager_featured);
            final AdapterFeatured adapter = new AdapterFeatured(getActivity(), list);
            final LinearLayout lyt_featured = root_view.findViewById(R.id.lyt_featured);
            view_pager_featured.setAdapter(adapter);
            view_pager_featured.setOffscreenPageLimit(4);

            if (list.size() > 0) {
                lyt_featured.setVisibility(View.VISIBLE);
            } else {
                lyt_featured.setVisibility(View.GONE);
            }

            view_pager_featured.addOnPageChangeListener(new ViewPager.SimpleOnPageChangeListener() {

                public void onPageSelected(int position) {
                    super.onPageSelected(position);
                    if (position < list.size()) {

                    }
                }
            });
            adapter.setOnItemClickListener((view, obj) -> {
                Intent intent = new Intent(getActivity(), ActivityRecipeDetail.class);
                intent.putExtra(Constant.EXTRA_OBJC, obj);
                startActivity(intent);
            });

            TabLayout tabLayout = root_view.findViewById(R.id.tabDots);
            tabLayout.setupWithViewPager(view_pager_featured, true);

            startAutoSlider(list.size());
        }

    }

    private void displayCategory(List<Category> list) {
        RecyclerView recyclerView = root_view.findViewById(R.id.recycler_view_category);
        recyclerView.setLayoutManager(new LinearLayoutManager(getActivity(), LinearLayoutManager.HORIZONTAL, false));
        AdapterHomeCategory adapterHomeCategory = new AdapterHomeCategory(getActivity(), list);
        recyclerView.setAdapter(adapterHomeCategory);
        recyclerView.setNestedScrollingEnabled(false);
        adapterHomeCategory.setOnItemClickListener((view, obj, position) -> {
            Intent intent = new Intent(getActivity(), ActivityCategoryDetail.class);
            intent.putExtra(EXTRA_OBJC, obj);
            startActivity(intent);
        });

        LinearLayout lyt_category = root_view.findViewById(R.id.lyt_category);
        if (list.size() > 0) {
            lyt_category.setVisibility(View.VISIBLE);
        } else {
            lyt_category.setVisibility(View.GONE);
        }
    }

    private void displayRecent(List<Recipe> list) {
        RecyclerView recyclerView = root_view.findViewById(R.id.recycler_view_recent);
        recyclerView.setLayoutManager(new StaggeredGridLayoutManager(1, StaggeredGridLayoutManager.HORIZONTAL));
        AdapterHomeRecipes adapterNews = new AdapterHomeRecipes(getActivity(), recyclerView, list);
        recyclerView.setAdapter(adapterNews);
        recyclerView.setNestedScrollingEnabled(false);
        adapterNews.setOnItemClickListener((view, obj, position) -> {
            Intent intent = new Intent(getActivity(), ActivityRecipeDetail.class);
            intent.putExtra(Constant.EXTRA_OBJC, obj);
            startActivity(intent);
        });

        LinearLayout lyt_recipes = root_view.findViewById(R.id.lyt_recipes);
        if (list.size() > 0) {
            lyt_recipes.setVisibility(View.VISIBLE);
        } else {
            lyt_recipes.setVisibility(View.GONE);
        }
    }

    private void displayVideos(List<Recipe> list) {
        RecyclerView recyclerView = root_view.findViewById(R.id.recycler_view_videos);
        recyclerView.setLayoutManager(new StaggeredGridLayoutManager(1, StaggeredGridLayoutManager.HORIZONTAL));
        AdapterHomeRecipes adapterNews = new AdapterHomeRecipes(getActivity(), recyclerView, list);
        recyclerView.setAdapter(adapterNews);
        recyclerView.setNestedScrollingEnabled(false);
        adapterNews.setOnItemClickListener((view, obj, position) -> {
            Intent intent = new Intent(getActivity(), ActivityRecipeDetail.class);
            intent.putExtra(Constant.EXTRA_OBJC, obj);
            startActivity(intent);
        });

        LinearLayout lyt_videos = root_view.findViewById(R.id.lyt_videos);
        if (list.size() > 0) {
            lyt_videos.setVisibility(View.VISIBLE);
        } else {
            lyt_videos.setVisibility(View.GONE);
        }
    }

    public void onDestroy() {
        if (!(callbackCall == null || callbackCall.isCanceled())) {
            this.callbackCall.cancel();
        }
        lyt_shimmer.stopShimmer();
        super.onDestroy();
    }

}

